/**
 * Copyright (C) 2010 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <glib.h>
#include <glib/gstdio.h>

#include "main.h"

#define ONBOOT_DIR LIBEXECDIR "/onboot"

GList *scripts = NULL;

gint
alphanum_sort (const gchar *a, const gchar *b)
{
    return (strcmp (a, b));
}

static void
exec_script (const gchar *script, gpointer user_data)
{
    gchar *path = NULL;

    path = g_strdup_printf ("%s/%s", ONBOOT_DIR, script);
    syslog (LOG_INFO, "onboot: executing command %s", path);
    g_spawn_command_line_async (path, NULL);
    g_free (path);
}

void
onboot_init ()
{
    GDir *d;
    const gchar *f_name;

    d = g_dir_open (ONBOOT_DIR, 0, NULL);
    if (! d)
        return;

    while ((f_name = g_dir_read_name (d)) != NULL)
        scripts = g_list_append (scripts, g_strdup (f_name));

    g_dir_close (d);

    scripts = g_list_sort (scripts, (GCompareFunc) alphanum_sort);

    g_list_foreach (scripts, (GFunc) exec_script, NULL);
}

void
onboot_reload ()
{
    /* no-op */
}

static void
item_free (gpointer data, gpointer user_data)
{
    g_free (data);
}

void
onboot_finalize ()
{
    g_list_foreach (scripts, item_free, NULL);
    g_list_free (scripts);
}

