/**
 * Copyright (C) 2010 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_TIME_H
#include <time.h>
#endif

#include <glib.h>
#include <glib/gstdio.h>

#include "main.h"

#define CONFFILE "/cdrom/etc/linutopd/scheduler/jobs"

static gchar *data;
static guint source_id;

gboolean
sched_parse_schedule (gpointer data)
{
    gchar **lines = NULL;
    gchar **linep;
    gchar **words = NULL;
    gchar *comment = NULL;
    guint hours, minutes;
    guint tmp;
    gchar *cmd = NULL;
    time_t t;
    struct tm *localt;

    time (&t);
    localt = localtime (&t);

    lines = g_strsplit ((gchar *)data, "\n", 0);
    for (linep = lines; *linep; linep++)
    {
        if ((comment = strstr (*linep, "#")) != NULL)
            *comment = '\0';

        if (strlen (*linep) < 1)
            continue;

        cmd = (gchar *) malloc (strlen (*linep) * sizeof (char));
        tmp = sscanf (*linep, "%d %d %s", &minutes, &hours, cmd);
        g_free (cmd); cmd = NULL;

        if (tmp != 3)
            continue;

        if ((hours > 23) || (hours < 0) || (minutes < 0) || (minutes > 59))
            continue;

        if ((hours != localt->tm_hour)
            || (minutes != localt->tm_min))
        {
            goto out;
        }

        words = g_strsplit (*linep, " ", 3);
        g_strstrip (words[2]);

        if (*words[2] == '/')
        {
            /* if the path is absolute we run the command */
            syslog (LOG_INFO, "scheduler: executing command %s", words[2]);
            g_spawn_command_line_async (words[2], NULL);
        }
        else
        {
            /*
             * otherwise we try to exec the script associated with a key
             * word
             */
            syslog (LOG_INFO, "scheduler: executing script %s", words[2]);
            cmd = g_strdup_printf (LIBEXECDIR "/scheduler/%s", words[2]);
            g_spawn_command_line_async (cmd, NULL);
            g_free (cmd); cmd = NULL;
        }

        g_strfreev (words);
    }

out:
    g_strfreev (lines);

    return TRUE;
}

void
scheduler_init ()
{
    if (! g_file_get_contents (CONFFILE, &data, NULL, NULL))
    {
        syslog (LOG_WARNING, "%s", "scheduler: unable to read configuration file");
        return;
    }

    /* run the GSourceFunc at startup */
    sched_parse_schedule (data);
    source_id = g_timeout_add_seconds (60, sched_parse_schedule, data);

    syslog (LOG_INFO, "%s", "scheduler: initialized");
}

void
scheduler_reload ()
{
    g_free (data); data = NULL;
    g_source_remove (source_id);
    if (! g_file_get_contents (CONFFILE, &data, NULL, NULL))
    {
        syslog (LOG_WARNING, "%s", "scheduler: unable to read configuration file");
        return;
    }

    syslog (LOG_INFO, "%s", "scheduler: reloaded configuration");
    sched_parse_schedule (data);
    source_id = g_timeout_add_seconds (60, sched_parse_schedule, data);
}

void
scheduler_finalize ()
{
    syslog (LOG_INFO, "%s", "scheduler: finalized");
    g_source_remove (source_id);
    g_free (data);
}

